# -*- coding: utf-8 -*-
"""
LimonAppDelegate.py - application delegate and main controller.


Copyright (C) 2005 Nir Soffer <nirs@freeshell.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
"""

from Foundation import *
from AppKit import *
from PyObjCTools import NibClassBuilder

import limonlib

# Stylesheet - probably there is a better way to define this

normalFontSize = 14 # should be user preference
normalFont = NSFont.systemFontOfSize_(normalFontSize)
smallFont = NSFont.systemFontOfSize_(normalFontSize * 0.85)

termAttributes = NSDictionary.alloc().initWithObjectsAndKeys_(
    normalFont, NSFontAttributeName,
    NSColor.blackColor(), NSForegroundColorAttributeName, None)

meaningAttributes = NSDictionary.alloc().initWithObjectsAndKeys_(
    smallFont, NSFontAttributeName,
    NSColor.blackColor(), NSForegroundColorAttributeName, None)

errorAttributes = NSDictionary.alloc().initWithObjectsAndKeys_(
    normalFont, NSFontAttributeName,
    NSColor.redColor(), NSForegroundColorAttributeName, None)

statusAttributes = NSDictionary.alloc().initWithObjectsAndKeys_(
    normalFont, NSFontAttributeName,
    NSColor.grayColor(), NSForegroundColorAttributeName, None)


class LimonAppDelegate(NibClassBuilder.AutoBaseClass):

    # ---------------------------------------------------------------------
    # Application delegate
        
    def awakeFromNib(self):
        self.entry.setStringValue_(u'')
        self.translation.setString_(u'')
        self.button.setEnabled_(False)

    # ---------------------------------------------------------------------
    # Defining terms

    def define_(self, sender):
        self.entry.selectText_(self)
        entry = self.entry.stringValue()
        if not entry:
            return
        
        self.progress.startAnimation_(self)
        textStorage = self.translation.textStorage()        
        textStorage.beginEditing()
        self._clear()        
        try:
            translation = limonlib.translate(entry.encode('utf-8'))
        except limonlib.TranslateError, err:
            # TODO: make better error message, maybe add link to the site?
            self._error(NSLocalizedString(u'Translation failed.', None))
        else:
            if not translation:
                # TODO: add link to the editor?
                self._status(NSLocalizedString(u'Undefined.', None))
            else:
                self._format(translation)        
        textStorage.endEditing()
        self.progress.stopAnimation_(self)

    def _format(self, translation):
        for d in translation:
            self._print(u'%(he)s   %(en)s\n' % d, termAttributes)
            if d['meaning']:
                self._print(u'%(meaning)s\n' % d, meaningAttributes)
            self._print(u'\n')
        # Use always right to left, regardles of the user interface 
        # language, becuase the meaning text is only in Hebrew.
        self._setDirection(NSWritingDirectionRightToLeft)
        
    def _error(self, message):        
        self._print(message, errorAttributes)
        # TODO: use current user interface language direction
        self._setDirection(NSWritingDirectionLeftToRight)
        
    def _status(self, message):
        self._print(message, statusAttributes)
        # TODO: use current user interface language direction
        self._setDirection(NSWritingDirectionLeftToRight)

    def _clear(self):
        textStorage = self.translation.textStorage()
        textStorage.deleteCharactersInRange_(
            NSMakeRange(0, textStorage.length()))

    def _print(self, string, attributes=None):
        textStorage = self.translation.textStorage()
        range = NSMakeRange(textStorage.length(), 0)
        textStorage.replaceCharactersInRange_withString_(range, string)
        range.length = len(string)
        textStorage.setAttributes_range_(attributes, range)

    def _setDirection(self, direction):
        textStorage = self.translation.textStorage()
        textStorage.addAttribute_value_range_(
            NSParagraphStyleAttributeName,
            self._paragraphStyleForDirection(direction),
            NSMakeRange(0, textStorage.length()))

    def _paragraphStyleForDirection(self, direction):
        style = NSParagraphStyle.defaultParagraphStyle().mutableCopy()
        style.setBaseWritingDirection_(direction)
        if direction == NSWritingDirectionRightToLeft:
            style.setAlignment_(NSRightTextAlignment)
        else:
            style.setAlignment_(NSLeftTextAlignment)
        return style
            
    # ---------------------------------------------------------------------
    # Entry field delegate
    
    def controlTextDidChange_(self, notification):
        if self.entry.stringValue().strip():
            self.button.setEnabled_(True)
        else:
            self.button.setEnabled_(False)

    def controlTextDidEndEditing_(self, notification):
        self.entry.setStringValue_(self.entry.stringValue().strip())

